﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Data.SqlClient;

namespace SQLskills_EFDemo
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            //DoSomething();
            //NPlusOneSelect();
            //LargeUnionAllList();
            //LargeInList();
            UseTVP();
        }


        public void DoSomething()
        {

            using (AdventureWorks2008Entities aw2008 = new AdventureWorks2008Entities())
            {

                var Accounts = from soh in aw2008.SalesOrderHeaders
                               select soh;
                /*
                foreach (var acct in Accounts)
                {
                    var Detail = from soh in aw2008.SalesOrderHeaders
                                 where soh.AccountNumber == acct.AccountNumber
                                 select soh.SalesOrderID;

                    foreach (var order in Detail)
                    {
                        var info = from sod in aw2008.SalesOrderDetails
                                   where sod.SalesOrderID == order
                                   select sod;

                        foreach (var detailrow in info)
                        {
                            int i = detailrow.OrderQty;
                        }
                    }

                }
                */


                // 1. Easy example but not very flexible 
                //    Select all products without any constraints 
                foreach (var product in aw2008.Products)
                {
                    Console.WriteLine(string.Format("{0} | {1} | {2}",
                        product.ProductID, product.Name, product.ListPrice));
                }

                // 2. Same example as above using LINQ 

                foreach (var product in from p in aw2008.Products
                                        select p)
                {
                    Console.WriteLine(string.Format("{0} | {1} | {2}",
                        product.ProductID, product.Name, product.ListPrice));

                }

                // 3. Same example as above externalizing the LINQ query 
                //    Select 20 products without any constraints 
                var query = from p in aw2008.Products
                            select p;

                foreach (var product in query.Take(20))
                {
                    Console.WriteLine(string.Format("{0} | {1} | {2}",
                        product.ProductID, product.Name, product.ListPrice));
                }

            }
        }


        public void LargeInList()
        {
            using (AdventureWorks2008Entities aw2008 = new AdventureWorks2008Entities())
            {
                var accountlist = from header in aw2008.SalesOrderHeaders
                                  select header.AccountNumber;

                string[] accounts = accountlist.Distinct().Take(20).ToArray<string>();

                var salesHeaders = from header in aw2008.SalesOrderHeaders
                                   where accounts.Contains(header.AccountNumber)
                                   select header;

                foreach (var salesHeader in salesHeaders)
                {
                    Console.WriteLine(salesHeader.AccountNumber);
                    Console.WriteLine(salesHeader.OrderDate);
                    Console.WriteLine(salesHeader.TotalDue);
                }

            }
        }

        public void UseTVP()
        {
            using (AdventureWorks2008Entities aw2008 = new AdventureWorks2008Entities())
            {
                var accountlist = from header in aw2008.SalesOrderHeaders
                                  select header.AccountNumber;

                DataTable accounts = new DataTable("Accounts");
                accounts.Columns.Add("AccountNumber", typeof(string));

                foreach (string account in accountlist.Distinct().Take(20).ToArray<string>())
                {
                    accounts.Rows.Add(account);
                }

                var acctsparam = new SqlParameter("Accounts", SqlDbType.Structured);
                acctsparam.Value = accounts;
                acctsparam.TypeName = "dbo.AccountList_TVP";


                var salesHeaders = aw2008.ExecuteStoreQuery<SalesOrderHeader>("EXECUTE GetSalesHeadersByAccountList @Accounts", acctsparam);

                foreach (var salesHeader in salesHeaders)
                {
                    Console.WriteLine(salesHeader.AccountNumber);
                    Console.WriteLine(salesHeader.OrderDate);
                    Console.WriteLine(salesHeader.TotalDue);
                }
            }

        }


        public void LargeUnionAllList()
        {
            using (AdventureWorks2008Entities aw2008 = new AdventureWorks2008Entities())
            {
                var accountlist = from header in aw2008.SalesOrderHeaders
                                  select header.AccountNumber;

                string[] accounts = accountlist.Distinct().Take(20).ToArray<string>();

                var salesHeaders = from header in aw2008.SalesOrderHeaders
                                   where accounts.Any(a => a == header.AccountNumber)
                                   select header;

                foreach (var salesHeader in salesHeaders)
                {
                    Console.WriteLine(salesHeader.AccountNumber);
                    Console.WriteLine(salesHeader.OrderDate);
                    Console.WriteLine(salesHeader.TotalDue);
                }
            }
        }

        public void NPlusOneSelect()
        {

             using (var aw2008 = new AdventureWorks2008Entities()) 
             {

                 var Accounts = from soh in aw2008.SalesOrderHeaders
                                select soh;

                 foreach (var acct in Accounts.Take(1))
                 {
                     var Details = from soh in aw2008.SalesOrderHeaders
                                   join sod in aw2008.SalesOrderDetails on soh.SalesOrderID equals sod.SalesOrderID
                                   where soh.AccountNumber == acct.AccountNumber
                                   select sod;

                         foreach (var detailrow in Details)
                         {
                             int i = detailrow.SpecialOfferID;
                         }
                 }
             }             
        }
    }
}


//public static class ObjectContextExt
//{
//    public static void ExecuteAccountListStoredProcedure(this ObjectContext context, string storeProcName, params object[] parameters)
//    {
//        string command = "EXECUTE " + storeProcName + " @Accounts";

//        context.ExecuteStoreCommand(command, parameters);
//    }

//}